package grabats.isstatic;

import DOM.Modifier;
import java.util.Arrays;
import org.eclipse.viatra2.emf.incquery.runtime.api.IPatternMatch;
import org.eclipse.viatra2.emf.incquery.runtime.api.impl.BasePatternMatch;
import org.eclipse.viatra2.patternlanguage.core.patternLanguage.Pattern;

/**
 * Pattern-specific match representation of the isStatic pattern, 
 * to be used in conjunction with IsStaticMatcher.
 * 
 * <p>Class fields correspond to parameters of the pattern. Fields with value null are considered unassigned.
 * Each instance is a (possibly partial) substitution of pattern parameters, 
 * usable to represent a match of the pattern in the result of a query, 
 * or to specify the bound (fixed) input parameters when issuing a query.
 * 
 * @see IsStaticMatcher
 * @see IsStaticProcessor
 * 
 */
public final class IsStaticMatch extends BasePatternMatch implements IPatternMatch {
  private Modifier fM;
  
  private static String[] parameterNames = {"M"};
  
  public IsStaticMatch(final Modifier pM) {
    this.fM = pM;
    
  }
  
  @Override
  public Object get(final String parameterName) {
    if ("M".equals(parameterName)) return this.fM;
    return null;
    
  }
  
  public Modifier getM() {
    return this.fM;
    
  }
  
  @Override
  public boolean set(final String parameterName, final Object newValue) {
    if ("M".equals(parameterName) && newValue instanceof DOM.Modifier) {
    	this.fM = (DOM.Modifier) newValue;
    	return true;
    }
    return false;
    
  }
  
  public void setM(final Modifier pM) {
    this.fM = pM;
    
  }
  
  @Override
  public String patternName() {
    return "isStatic";
    
  }
  
  @Override
  public String[] parameterNames() {
    return IsStaticMatch.parameterNames;
    
  }
  
  @Override
  public Object[] toArray() {
    return new Object[]{fM};
    
  }
  
  @Override
  public String prettyPrint() {
    StringBuilder result = new StringBuilder();
    result.append("\"M\"=" + prettyPrintValue(fM)
    );return result.toString();
    
  }
  
  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((fM == null) ? 0 : fM.hashCode()); 
    return result; 
    
  }
  
  @Override
  public boolean equals(final Object obj) {
    if (this == obj)
    	return true;
    if (obj == null)
    	return false;
    if (!(obj instanceof IPatternMatch))
    	return false;
    IPatternMatch otherSig  = (IPatternMatch) obj;
    if (!pattern().equals(otherSig.pattern()))
    	return false;
    if (!IsStaticMatch.class.equals(obj.getClass()))
    	return Arrays.deepEquals(toArray(), otherSig.toArray());
    				IsStaticMatch other = (IsStaticMatch) obj;
    				if (fM == null) {if (other.fM != null) return false;}
    				else if (!fM.equals(other.fM)) return false;
    				return true;
    
  }
  
  @Override
  public Pattern pattern() {
    return IsStaticMatcher.FACTORY.getPattern();
  }
}
