package grabats.methodoftype;

import DOM.MethodDeclaration;
import DOM.TypeDeclaration;
import java.util.Arrays;
import org.eclipse.viatra2.emf.incquery.runtime.api.IPatternMatch;
import org.eclipse.viatra2.emf.incquery.runtime.api.impl.BasePatternMatch;
import org.eclipse.viatra2.patternlanguage.core.patternLanguage.Pattern;

/**
 * Pattern-specific match representation of the methodOfType pattern, 
 * to be used in conjunction with MethodOfTypeMatcher.
 * 
 * <p>Class fields correspond to parameters of the pattern. Fields with value null are considered unassigned.
 * Each instance is a (possibly partial) substitution of pattern parameters, 
 * usable to represent a match of the pattern in the result of a query, 
 * or to specify the bound (fixed) input parameters when issuing a query.
 * 
 * @see MethodOfTypeMatcher
 * @see MethodOfTypeProcessor
 * 
 */
public final class MethodOfTypeMatch extends BasePatternMatch implements IPatternMatch {
  private TypeDeclaration fT;
  
  private MethodDeclaration fM;
  
  private static String[] parameterNames = {"T", "M"};
  
  public MethodOfTypeMatch(final TypeDeclaration pT, final MethodDeclaration pM) {
    this.fT = pT;
    this.fM = pM;
    
  }
  
  @Override
  public Object get(final String parameterName) {
    if ("T".equals(parameterName)) return this.fT;
    if ("M".equals(parameterName)) return this.fM;
    return null;
    
  }
  
  public TypeDeclaration getT() {
    return this.fT;
    
  }
  
  public MethodDeclaration getM() {
    return this.fM;
    
  }
  
  @Override
  public boolean set(final String parameterName, final Object newValue) {
    if ("T".equals(parameterName) && newValue instanceof DOM.TypeDeclaration) {
    	this.fT = (DOM.TypeDeclaration) newValue;
    	return true;
    }
    if ("M".equals(parameterName) && newValue instanceof DOM.MethodDeclaration) {
    	this.fM = (DOM.MethodDeclaration) newValue;
    	return true;
    }
    return false;
    
  }
  
  public void setT(final TypeDeclaration pT) {
    this.fT = pT;
    
  }
  
  public void setM(final MethodDeclaration pM) {
    this.fM = pM;
    
  }
  
  @Override
  public String patternName() {
    return "methodOfType";
    
  }
  
  @Override
  public String[] parameterNames() {
    return MethodOfTypeMatch.parameterNames;
    
  }
  
  @Override
  public Object[] toArray() {
    return new Object[]{fT, fM};
    
  }
  
  @Override
  public String prettyPrint() {
    StringBuilder result = new StringBuilder();
    result.append("\"T\"=" + prettyPrintValue(fT) + ", ");
    
    result.append("\"M\"=" + prettyPrintValue(fM)
    );return result.toString();
    
  }
  
  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((fT == null) ? 0 : fT.hashCode()); 
    result = prime * result + ((fM == null) ? 0 : fM.hashCode()); 
    return result; 
    
  }
  
  @Override
  public boolean equals(final Object obj) {
    if (this == obj)
    	return true;
    if (obj == null)
    	return false;
    if (!(obj instanceof IPatternMatch))
    	return false;
    IPatternMatch otherSig  = (IPatternMatch) obj;
    if (!pattern().equals(otherSig.pattern()))
    	return false;
    if (!MethodOfTypeMatch.class.equals(obj.getClass()))
    	return Arrays.deepEquals(toArray(), otherSig.toArray());
    				MethodOfTypeMatch other = (MethodOfTypeMatch) obj;
    				if (fT == null) {if (other.fT != null) return false;}
    				else if (!fT.equals(other.fT)) return false;
    				if (fM == null) {if (other.fM != null) return false;}
    				else if (!fM.equals(other.fM)) return false;
    				return true;
    
  }
  
  @Override
  public Pattern pattern() {
    return MethodOfTypeMatcher.FACTORY.getPattern();
  }
}
