package grabats.methodwithreturntype;

import DOM.MethodDeclaration;
import java.util.Arrays;
import org.eclipse.viatra2.emf.incquery.runtime.api.IPatternMatch;
import org.eclipse.viatra2.emf.incquery.runtime.api.impl.BasePatternMatch;
import org.eclipse.viatra2.patternlanguage.core.patternLanguage.Pattern;

/**
 * Pattern-specific match representation of the methodWithReturnType pattern, 
 * to be used in conjunction with MethodWithReturnTypeMatcher.
 * 
 * <p>Class fields correspond to parameters of the pattern. Fields with value null are considered unassigned.
 * Each instance is a (possibly partial) substitution of pattern parameters, 
 * usable to represent a match of the pattern in the result of a query, 
 * or to specify the bound (fixed) input parameters when issuing a query.
 * 
 * @see MethodWithReturnTypeMatcher
 * @see MethodWithReturnTypeProcessor
 * 
 */
public final class MethodWithReturnTypeMatch extends BasePatternMatch implements IPatternMatch {
  private MethodDeclaration fM;
  
  private String fS;
  
  private static String[] parameterNames = {"M", "S"};
  
  public MethodWithReturnTypeMatch(final MethodDeclaration pM, final String pS) {
    this.fM = pM;
    this.fS = pS;
    
  }
  
  @Override
  public Object get(final String parameterName) {
    if ("M".equals(parameterName)) return this.fM;
    if ("S".equals(parameterName)) return this.fS;
    return null;
    
  }
  
  public MethodDeclaration getM() {
    return this.fM;
    
  }
  
  public String getS() {
    return this.fS;
    
  }
  
  @Override
  public boolean set(final String parameterName, final Object newValue) {
    if ("M".equals(parameterName) && newValue instanceof DOM.MethodDeclaration) {
    	this.fM = (DOM.MethodDeclaration) newValue;
    	return true;
    }
    if ("S".equals(parameterName) && newValue instanceof java.lang.String) {
    	this.fS = (java.lang.String) newValue;
    	return true;
    }
    return false;
    
  }
  
  public void setM(final MethodDeclaration pM) {
    this.fM = pM;
    
  }
  
  public void setS(final String pS) {
    this.fS = pS;
    
  }
  
  @Override
  public String patternName() {
    return "methodWithReturnType";
    
  }
  
  @Override
  public String[] parameterNames() {
    return MethodWithReturnTypeMatch.parameterNames;
    
  }
  
  @Override
  public Object[] toArray() {
    return new Object[]{fM, fS};
    
  }
  
  @Override
  public String prettyPrint() {
    StringBuilder result = new StringBuilder();
    result.append("\"M\"=" + prettyPrintValue(fM) + ", ");
    
    result.append("\"S\"=" + prettyPrintValue(fS)
    );return result.toString();
    
  }
  
  @Override
  public int hashCode() {
    final int prime = 31;
    int result = 1;
    result = prime * result + ((fM == null) ? 0 : fM.hashCode()); 
    result = prime * result + ((fS == null) ? 0 : fS.hashCode()); 
    return result; 
    
  }
  
  @Override
  public boolean equals(final Object obj) {
    if (this == obj)
    	return true;
    if (obj == null)
    	return false;
    if (!(obj instanceof IPatternMatch))
    	return false;
    IPatternMatch otherSig  = (IPatternMatch) obj;
    if (!pattern().equals(otherSig.pattern()))
    	return false;
    if (!MethodWithReturnTypeMatch.class.equals(obj.getClass()))
    	return Arrays.deepEquals(toArray(), otherSig.toArray());
    				MethodWithReturnTypeMatch other = (MethodWithReturnTypeMatch) obj;
    				if (fM == null) {if (other.fM != null) return false;}
    				else if (!fM.equals(other.fM)) return false;
    				if (fS == null) {if (other.fS != null) return false;}
    				else if (!fS.equals(other.fS)) return false;
    				return true;
    
  }
  
  @Override
  public Pattern pattern() {
    return MethodWithReturnTypeMatcher.FACTORY.getPattern();
  }
}
